Attribute VB_Name = "mMap"
Option Explicit

'Types
    Public Type tMap
        w As Long 'W stands for width
        h As Long 'H stands for height
        
        Floor() As Long
        Roof() As Long
    End Type

'Variables
    Public Map As tMap

Public Sub CreateMap(iW As Long, iH As Long)
    With Map
        'Get size
        .w = iW
        .h = iH
        
        'Create map
        ReDim .Floor(.w * .h)
        ReDim .Roof(.w * .h)
    End With
End Sub

Public Sub Reset()
    'Reset map
    Map.w = 0
    Map.h = 0
    
    Erase Map.Floor
    Erase Map.Roof
End Sub

Public Sub DrawFloor()
    Dim X As Long
    Dim Y As Long
    
    Dim Index As Long
    Dim TileIndex As Long
    
    Dim TempX As Single
    Dim TempY As Single
    
    With Map
        'Get map position
        TempX = (Camera.X / TileSize)
        TempY = (Camera.Y / TileSize)
        
        'Calculate index of this positon
        Index = Int(TempY) * .w + Int(TempX)
        
        'Calculate offset
        TempX = TempX - Int(TempX)
        TempY = TempY - Int(TempY)
        
        'Draw the map
        For Y = 0 To Camera.TilesY
            For X = 0 To Camera.TilesX
                'Check index
                If Index > (Map.w * Map.h) Then: Exit Sub
                
                'Get tile index
                TileIndex = .Floor(Index)
                
                'Draw tile
                BitBlt Camera.FrontDC, _
                    (X - TempX) * TileSize, (Y - TempY) * TileSize, _
                    TileSize, TileSize, _
                    Bitmap(Tile(TileIndex).Bitmap).Picture.DC, _
                    Tile(TileIndex).ActFrame * TileSize, 0, _
                    vbSrcCopy
                
                'Next tile
                Index = Index + 1
            Next
            
            'Get next line start
            Index = Index + .w - Camera.TilesX - 1
        Next
    End With
End Sub

Public Sub DrawRoof()
    Dim X As Long
    Dim Y As Long
    
    Dim Index As Long
    Dim TileIndex As Long
    
    Dim TempX As Single
    Dim TempY As Single
    
    With Map
        'Get map position
        TempX = (Camera.X / TileSize)
        TempY = (Camera.Y / TileSize)
        
        'Calculate index of this positon
        Index = Int(TempY) * .w + Int(TempX)
        
        'Calculate offset
        TempX = TempX - Int(TempX)
        TempY = TempY - Int(TempY)
        
        'Draw the map
        For Y = 0 To Camera.TilesY
            For X = 0 To Camera.TilesX
                'Check index
                If Index > (Map.w * Map.h) Then: Exit Sub
                
                'Get tile index
                TileIndex = .Roof(Index)
                
                If TileIndex > 0 Then
                    'Draw tile mask
                    BitBlt Camera.FrontDC, _
                        (X - TempX) * TileSize, (Y - TempY) * TileSize, _
                        TileSize, TileSize, _
                        Bitmap(Tile(TileIndex).Bitmap).Mask.DC, _
                        Tile(TileIndex).ActFrame * TileSize, 0, _
                        vbSrcPaint 'logical AND
                    
                    'Draw roof tile
                    BitBlt Camera.FrontDC, _
                        (X - TempX) * TileSize, (Y - TempY) * TileSize, _
                        TileSize, TileSize, _
                        Bitmap(Tile(TileIndex).Bitmap).Picture.DC, _
                        Tile(TileIndex).ActFrame * TileSize, 0, _
                        vbSrcAnd 'logical OR
                End If
                
                'Next tile
                Index = Index + 1
            Next
            
            'Get next line start
            Index = Index + .w - Camera.TilesX - 1
        Next
    End With
End Sub


Public Function SaveData(iFileName As String) As Boolean
On Error GoTo 1

    Dim FileNumber As Integer
    
    'Open the specified file
    FileNumber = FreeFile
    Open iFileName For Binary As FileNumber
        'Write data
        WriteData FileNumber
    Close FileNumber
    
    'We're done
    SaveData = True
    Exit Function

1  'Catched an error
    SaveData = False
End Function

Public Function LoadData(iFileName As String) As Boolean
On Error GoTo 1

    Dim FileNumber As Integer
    
    'Check existance
    If Dir(iFileName, vbNormal) = "" Then: GoTo 1
    
    'Open the specified file
    FileNumber = FreeFile
    Open iFileName For Binary As FileNumber
        'Read data
        ReadData FileNumber
    Close FileNumber
    
    'We're done
    LoadData = True
    Exit Function

1  'Catched an error
    LoadData = False
End Function

Public Sub SetFloorTile(iX As Long, iY As Long, Index As Long)
    'Set tile at the specified 0-based position
    Map.Floor(iY * Map.w + iX) = Index
End Sub

Public Sub SetRoofTile(iX As Long, iY As Long, Index As Long)
    'Set tile at the specified 0-based position
    Map.Roof(iY * Map.w + iX) = Index
End Sub
Public Sub WriteData(iFileNumber As Integer)
    'Write tile set
    mTiles.WriteData iFileNumber
    
    With Map
        'Write map size
        Put iFileNumber, , .w
        Put iFileNumber, , .h
        
        'Write map data
        Put iFileNumber, , .Floor
        Put iFileNumber, , .Roof
    End With
End Sub

Public Sub ReadData(iFileNumber As Integer)
    'Read tile set
    mTiles.ReadData iFileNumber
        
    With Map
        'Read m size
        Get iFileNumber, , .w
        Get iFileNumber, , .h
        
        'Allocate memory
        ReDim .Floor(.w * .h)
        ReDim .Roof(.w * .h)
        
        'Read map data
        Get iFileNumber, , .Floor
        Get iFileNumber, , .Roof
    End With
End Sub



